@@ -402,7 +402,7 @@ Plm_image::load_native_dicom (const char* fname)
 #if PLM_DCM_USE_DCMTK
 
     Dcmtk_rt_study drs (fname);
-    drs.parse_directory ();
+    drs.load_directory ();
     this->set (drs.get_image ());
 
     /* GCS FIX: Here we need to set the metadata loaded from 
@@ -443,6 +443,7 @@ Plm_image::save_short_dicom (
     Rt_study_metadata *rsm
 )
 {
+#if defined (commentout)
     switch (this->m_type) {
     case PLM_IMG_TYPE_ITK_UCHAR:
 	itk_image_save_short_dicom (this->m_itk_uchar, fname, rsm);
@@ -481,6 +482,18 @@ Plm_image::save_short_dicom (
 			" (type = %d)\n", this->m_type);
 	break;
     }
+#endif
+    
+#if PLM_DCM_USE_DCMTK
+    Dcmtk_rt_study drs;
+    Rt_study_metadata::Pointer this_rsm = Rt_study_metadata::New (rsm);
+    drs.set_rt_study_metadata (this_rsm);
+    Plm_image::Pointer this_image = Plm_image::New (this);
+    drs.set_image (this_image);
+    drs.save (fname);
+#else
+    print_and_exit ("Error: tried to save dicom without DCMTK\n");
+#endif
 }
 
 void
@@ -929,6 +942,15 @@ void
 Plm_image::convert_to_itk_short (void)
 {
     switch (this->m_type) {
+    case PLM_IMG_TYPE_ITK_UCHAR:
+	CONVERT_ITK_ITK (short, uchar);
+	return;
+    case PLM_IMG_TYPE_ITK_CHAR:
+	CONVERT_ITK_ITK (short, char);
+	return;
+    case PLM_IMG_TYPE_ITK_USHORT:
+	CONVERT_ITK_ITK (short, ushort);
+	break;
     case PLM_IMG_TYPE_ITK_SHORT:
 	return;
     case PLM_IMG_TYPE_ITK_ULONG:
@@ -940,6 +962,9 @@ Plm_image::convert_to_itk_short (void)
     case PLM_IMG_TYPE_ITK_FLOAT:
 	CONVERT_ITK_ITK (short, float);
 	break;
+    case PLM_IMG_TYPE_ITK_DOUBLE:
+	CONVERT_ITK_ITK (short, double);
+	return;
     case PLM_IMG_TYPE_GPUIT_SHORT:
 	this->m_itk_short = this->convert_gpuit_to_itk<
             ShortImageType::Pointer, short> (this->get_vol());
