@@ -419,7 +419,7 @@ load_gpuit_bsp (Xform *xf, const char* fn)
 
 template< class T >
 static void
-itk_xform_save (T transform, const char *filename)
+itk_xform_save (const T transform, const char *filename)
 {
     typedef itk::TransformFileWriter TransformWriterType;
     TransformWriterType::Pointer outputTransformWriter;
@@ -524,7 +524,7 @@ itk_xform_load (Xform *xf, const char* fn)
 }
 
 void
-Xform::save_gpuit_vf (const char* fn)
+Xform::save_gpuit_vf (const char* fn) const
 {
     //write_mha (fn, this->get_gpuit_vf().get());
 
@@ -534,7 +534,7 @@ Xform::save_gpuit_vf (const char* fn)
 }
 
 void
-Xform::save (const char* fn)
+Xform::save (const char* fn) const
 {
     switch (this->m_type) {
     case XFORM_ITK_TRANSLATION:
@@ -574,7 +574,7 @@ Xform::save (const char* fn)
 }
 
 void
-Xform::save (const std::string& fn)
+Xform::save (const std::string& fn) const
 {
     this->save (fn.c_str());
 }
@@ -1582,7 +1582,8 @@ Volume::Pointer
 xform_itk_vf_to_gpuit_vf (
     DeformationFieldType::Pointer itk_vf, const Plm_image_header *pih)
 {
-    Volume_header vh (pih);
+    const Plm_image_header pih_vf (itk_vf);
+    Volume_header vh (&pih_vf);
     Volume::Pointer vf_out = Volume::New (vh, PT_VF_FLOAT_INTERLEAVED, 3);
     float* img = (float*) vf_out->img;
     FloatVector3DType displacement;
@@ -1596,6 +1597,10 @@ xform_itk_vf_to_gpuit_vf (
             img[i++] = displacement[r];
         }
     }
+
+    if (pih) {
+        /* GCS FIX: Resample to desired resolution */
+    }
     return vf_out;
 }
 
@@ -2132,6 +2137,16 @@ Xform::get_grid_spacing (float grid_spacing[3])
     }
 }
 
+bool
+Xform::is_linear ()
+{
+    return this->m_type == XFORM_ITK_TRANSLATION
+        || this->m_type == XFORM_ITK_VERSOR
+        || this->m_type == XFORM_ITK_QUATERNION
+        || this->m_type == XFORM_ITK_SIMILARITY
+        || this->m_type == XFORM_ITK_AFFINE;
+}
+
 void
 Xform::print ()
 {
