@@ -24,17 +24,17 @@ class Xform;
 class Xform_private;
 
 enum Xform_type {
-    XFORM_NONE                  = 0,
-    XFORM_ITK_TRANSLATION       = 1,
-    XFORM_ITK_VERSOR            = 2,
-    XFORM_ITK_QUATERNION        = 3,
-    XFORM_ITK_AFFINE            = 4,
-    XFORM_ITK_BSPLINE           = 5,
-    XFORM_ITK_TPS               = 6,
-    XFORM_ITK_VECTOR_FIELD      = 7,
-    XFORM_GPUIT_BSPLINE         = 8,
-    XFORM_GPUIT_VECTOR_FIELD    = 9,
-    XFORM_ITK_SIMILARITY        = 10
+    XFORM_NONE,
+    XFORM_ITK_TRANSLATION,
+    XFORM_ITK_VERSOR,
+    XFORM_ITK_QUATERNION,
+    XFORM_ITK_SIMILARITY,
+    XFORM_ITK_AFFINE,
+    XFORM_ITK_BSPLINE,
+    XFORM_ITK_TPS,
+    XFORM_ITK_VECTOR_FIELD,
+    XFORM_GPUIT_BSPLINE,
+    XFORM_GPUIT_VECTOR_FIELD,
 };
 
 /* itk basic transforms */
@@ -58,6 +58,11 @@ typedef itk::ThinPlateSplineKernelTransform <
 typedef DoubleTpsTransformType TpsTransformType;
 
 
+/*! \brief 
+ * The Xform class is an abstraction that encapsulates a simple transform, 
+ * either native format (B-spline or vector field), or ITK format 
+ * (linear, B-spline, or vector field).
+ */
 class PLMBASE_API Xform {
 public:
     SMART_POINTER_SUPPORT (Xform);
@@ -85,8 +90,8 @@ public:
 
     void load (const char* fn);
     void load (const std::string& fn);
-    void save (const char* fn);
-    void save (const std::string& fn);
+    void save (const char* fn) const;
+    void save (const std::string& fn) const;
 
     TranslationTransformType::Pointer get_trn () const;
     VersorTransformType::Pointer get_vrs () const;
@@ -128,10 +133,14 @@ public:
     Plm_image_header get_plm_image_header ();
     void get_grid_spacing (float grid_spacing[3]);
 
+    /*! \brief Return true if the xform type is translation, rigid, 
+      similarity, or affine. */
+    bool is_linear ();
+
     void print ();
 
 protected:
-    void save_gpuit_vf (const char* fn);
+    void save_gpuit_vf (const char* fn) const;
 
 public:
     Xform& operator= (const Xform& xf);
