@@ -8,24 +8,26 @@
 
 #include "xio_studyset.h"
 
+class Rtss;
 class Metadata;
 class Xio_studyset;
 
 PLMBASE_C_API void xio_structures_load (
-        Rtss_structure_set *structures,
+        Rtss *structures,
         const Xio_studyset& xsl
 );
 
-PLMBASE_C_API void xio_structures_save (
-        Rtss_structure_set *cxt, 
-        Metadata *meta,
-        Xio_ct_transform *transform, 
-        Xio_version xio_version, 
-        const char *output_dir
+PLMBASE_API void
+xio_structures_save (
+    const Rt_study_metadata::Pointer& rsm,
+    Rtss *cxt, 
+    Xio_ct_transform *transform, 
+    Xio_version xio_version, 
+    const char *output_dir
 );
 
 PLMBASE_C_API void xio_structures_apply_transform (
-        Rtss_structure_set *structures,
+        Rtss *structures,
         Xio_ct_transform *transform
 );
 
