@@ -44,9 +44,9 @@
 #include "pcmd_threshold.h"
 #include "pcmd_thumbnail.h"
 #include "pcmd_union.h"
-#include "pcmd_vf_invert.h"
 #include "pcmd_warp.h"
 #include "pcmd_xf_convert.h"
+#include "pcmd_xf_invert.h"
 #include "pcmd_xio_dvh.h"
 #include "plm_exception.h"
 #include "plm_version.h"
@@ -111,10 +111,10 @@ print_usage (int return_code)
         "\n"
         "  thumbnail   "
         "  union       "
-        "  vf-invert   "
         "  warp        "
 //        "  xio-dvh     "
         "  xf-convert  "
+        "  xf-invert   "
         "\n"
         "\n"
         "For detailed usage of a specific command, type:\n"
@@ -269,9 +269,6 @@ do_command (int argc, char* argv[])
     else if (!strcmp (command, "union")) {
         do_command_union (argc, argv);
     }
-    else if (!strcmp (command, "vf-invert")) {
-        do_command_vf_invert (argc, argv);
-    }
     else if (!strcmp (command, "warp")) {
         /* convert and warp are the same */
         do_command_warp (argc, argv);
@@ -279,6 +276,11 @@ do_command (int argc, char* argv[])
     else if (!strcmp (command, "xf-convert")) {
         do_command_xf_convert (argc, argv);
     }
+    else if (!strcmp (command, "vf-invert")
+        || !strcmp (command, "xf-invert"))
+    {
+        do_command_vf_invert (argc, argv);
+    }
     else if (!strcmp (command, "xio-dvh")) {
         do_command_xio_dvh (argc, argv);
     }
