@@ -11,16 +11,19 @@
 #include <omp.h>
 #endif
 
-#include "plmbase.h"
-#include "plmreconstruct.h"
-#include "plmsys.h"
-#include "plmutil.h"
-
+#include "bowtie_correction.h"
+#include "delayload.h"
+#include "fdk.h"
 #include "fdk_cuda.h"
 #include "fdk_opencl.h"
-
-#include "delayload.h"
+#include "file_util.h"
 #include "plm_math.h"
+#include "plm_timer.h"
+#include "proj_image.h"
+#include "proj_image_dir.h"
+#include "proj_image_filter.h"
+#include "proj_matrix.h"
+#include "volume.h"
 
 #if CUDA_FOUND
 void
@@ -178,6 +181,7 @@ get_pixel_value_c (Proj_image* cbi, double r, double c)
 {
     int rr, cc;
 
+#if defined (commentout)
     r += 0.5;
     if (r < 0) return 0.0;
     rr = (int) r;
@@ -187,6 +191,17 @@ get_pixel_value_c (Proj_image* cbi, double r, double c)
     if (c < 0) return 0.0;
     cc = (int) c;
     if (cc >= cbi->dim[0]) return 0.0;
+#endif
+
+    r += 0.5;
+    if (r < 0) return 0.0;
+    if (r >= (double) cbi->dim[1]) return 0.0;
+    rr = (int) r;
+
+    c += 0.5;
+    if (c < 0) return 0.0;
+    if (c >= (double) cbi->dim[0]) return 0.0;
+    cc = (int) c;
 
     return cbi->img[rr*cbi->dim[0] + cc];
 }
@@ -231,7 +246,7 @@ project_volume_onto_image_d (Volume* vol, Proj_image* cbi, float scale)
     /* Precompute partial projections here */
 #pragma omp parallel for
     for (i = 0; i < vol->dim[0]; i++) {
-	double x = (double) (vol->offset[0] + i * vol->spacing[0]);
+	double x = (double) (vol->origin[0] + i * vol->spacing[0]);
 	xip[i*3+0] = x * (pmat->matrix[0] + pmat->ic[0] * pmat->matrix[8]);
 	xip[i*3+1] = x * (pmat->matrix[4] + pmat->ic[1] * pmat->matrix[8]);
 	xip[i*3+2] = x * pmat->matrix[8];
@@ -239,7 +254,7 @@ project_volume_onto_image_d (Volume* vol, Proj_image* cbi, float scale)
     
 #pragma omp parallel for
     for (j = 0; j < vol->dim[1]; j++) {
-	double y = (double) (vol->offset[1] + j * vol->spacing[1]);
+	double y = (double) (vol->origin[1] + j * vol->spacing[1]);
 	yip[j*3+0] = y * (pmat->matrix[1] + pmat->ic[0] * pmat->matrix[9]);
 	yip[j*3+1] = y * (pmat->matrix[5] + pmat->ic[1] * pmat->matrix[9]);
 	yip[j*3+2] = y * pmat->matrix[9];
@@ -247,7 +262,7 @@ project_volume_onto_image_d (Volume* vol, Proj_image* cbi, float scale)
 
 #pragma omp parallel for
     for (k = 0; k < vol->dim[2]; k++) {
-	double z = (double) (vol->offset[2] + k * vol->spacing[2]);
+	double z = (double) (vol->origin[2] + k * vol->spacing[2]);
 	zip[k*3+0] = z * (pmat->matrix[2] + pmat->ic[0] * pmat->matrix[10]) 
 		+ pmat->ic[0] * pmat->matrix[11] + pmat->matrix[3];
 	zip[k*3+1] = z * (pmat->matrix[6] + pmat->ic[1] * pmat->matrix[10]) 
@@ -324,19 +339,19 @@ project_volume_onto_image_c (Volume* vol, Proj_image* cbi, float scale)
 
     /* Precompute partial projections here */
     for (i = 0; i < vol->dim[0]; i++) {
-	double x = (double) (vol->offset[0] + i * vol->spacing[0]);
+	double x = (double) (vol->origin[0] + i * vol->spacing[0]);
 	xip[i*3+0] = x * (pmat->matrix[0] + pmat->ic[0] * pmat->matrix[8]);
 	xip[i*3+1] = x * (pmat->matrix[4] + pmat->ic[1] * pmat->matrix[8]);
 	xip[i*3+2] = x * pmat->matrix[8];
     }
     for (j = 0; j < vol->dim[1]; j++) {
-	double y = (double) (vol->offset[1] + j * vol->spacing[1]);
+	double y = (double) (vol->origin[1] + j * vol->spacing[1]);
 	yip[j*3+0] = y * (pmat->matrix[1] + pmat->ic[0] * pmat->matrix[9]);
 	yip[j*3+1] = y * (pmat->matrix[5] + pmat->ic[1] * pmat->matrix[9]);
 	yip[j*3+2] = y * pmat->matrix[9];
     }
     for (k = 0; k < vol->dim[2]; k++) {
-	double z = (double) (vol->offset[2] + k * vol->spacing[2]);
+	double z = (double) (vol->origin[2] + k * vol->spacing[2]);
 	zip[k*3+0] = z * (pmat->matrix[2] + pmat->ic[0] * pmat->matrix[10]) 
 		+ pmat->ic[0] * pmat->matrix[11] + pmat->matrix[3];
 	zip[k*3+1] = z * (pmat->matrix[6] + pmat->ic[1] * pmat->matrix[10]) 
@@ -399,21 +414,21 @@ project_volume_onto_image_b (Volume* vol, Proj_image* cbi, float scale)
 
     /* Precompute partial projections here */
     for (i = 0; i < vol->dim[0]; i++) {
-	x[i] = (double) (vol->offset[0] + i * vol->spacing[0]);
+	x[i] = (double) (vol->origin[0] + i * vol->spacing[0]);
 	xip[i*3+0] = x[i] * pmat->matrix[0];
 	xip[i*3+1] = x[i] * pmat->matrix[4];
 	xip[i*3+2] = x[i] * pmat->matrix[8];
 	x[i] *= pmat->nrm[0];
     }
     for (j = 0; j < vol->dim[1]; j++) {
-	y[j] = (double) (vol->offset[1] + j * vol->spacing[1]);
+	y[j] = (double) (vol->origin[1] + j * vol->spacing[1]);
 	yip[j*3+0] = y[j] * pmat->matrix[1];
 	yip[j*3+1] = y[j] * pmat->matrix[5];
 	yip[j*3+2] = y[j] * pmat->matrix[9];
 	y[j] *= pmat->nrm[1];
     }
     for (k = 0; k < vol->dim[2]; k++) {
-	z[k] = (double) (vol->offset[2] + k * vol->spacing[2]);
+	z[k] = (double) (vol->origin[2] + k * vol->spacing[2]);
 	zip[k*3+0] = z[k] * pmat->matrix[2];
 	zip[k*3+1] = z[k] * pmat->matrix[6];
 	zip[k*3+2] = z[k] * pmat->matrix[10];
@@ -473,21 +488,21 @@ project_volume_onto_image_a (Volume* vol, Proj_image* cbi, float scale)
 
     /* Precompute partial projections here */
     for (i = 0; i < vol->dim[0]; i++) {
-	x[i] = (double) (vol->offset[0] + i * vol->spacing[0]);
+	x[i] = (double) (vol->origin[0] + i * vol->spacing[0]);
 	xip[i*3+0] = x[i] * pmat->matrix[0];
 	xip[i*3+1] = x[i] * pmat->matrix[4];
 	xip[i*3+2] = x[i] * pmat->matrix[8];
 	x[i] *= pmat->nrm[0];
     }
     for (j = 0; j < vol->dim[1]; j++) {
-	y[j] = (double) (vol->offset[1] + j * vol->spacing[1]);
+	y[j] = (double) (vol->origin[1] + j * vol->spacing[1]);
 	yip[j*3+0] = y[j] * pmat->matrix[1];
 	yip[j*3+1] = y[j] * pmat->matrix[5];
 	yip[j*3+2] = y[j] * pmat->matrix[9];
 	y[j] *= pmat->nrm[1];
     }
     for (k = 0; k < vol->dim[2]; k++) {
-	z[k] = (double) (vol->offset[2] + k * vol->spacing[2]);
+	z[k] = (double) (vol->origin[2] + k * vol->spacing[2]);
 	zip[k*3+0] = z[k] * pmat->matrix[2];
 	zip[k*3+1] = z[k] * pmat->matrix[6];
 	zip[k*3+2] = z[k] * pmat->matrix[10];
@@ -545,14 +560,14 @@ project_volume_onto_image_reference (
     vp[3] = 1.0;
     /* Loop k (slices), j (rows), i (cols) */
     for (k = 0; k < vol->dim[2]; k++) {
-	vp[2] = (double) (vol->offset[2] + k * vol->spacing[2]);
+	vp[2] = (double) (vol->origin[2] + k * vol->spacing[2]);
 	for (j = 0; j < vol->dim[1]; j++) {
-	    vp[1] = (double) (vol->offset[1] + j * vol->spacing[1]);
+	    vp[1] = (double) (vol->origin[1] + j * vol->spacing[1]);
 	    for (i = 0; i < vol->dim[0]; i++) {
 		double ip[3];        /* ip = image position */
 		double s;            /* s = projection of vp onto s axis */
-		vp[0] = (double) (vol->offset[0] + i * vol->spacing[0]);
-		mat43_mult_vec3 (ip, pmat->matrix, vp);  /* ip = matrix * vp */
+		vp[0] = (double) (vol->origin[0] + i * vol->spacing[0]);
+		mat43_mult_vec4 (ip, pmat->matrix, vp);  /* ip = matrix * vp */
 		ip[0] = pmat->ic[0] + ip[0] / ip[2];
 		ip[1] = pmat->ic[1] + ip[1] / ip[2];
 		/* Distance on cenral axis from voxel center to source */
