@@ -4,33 +4,27 @@
 #ifndef _dir_list_h_
 #define _dir_list_h_
 
-/**
-*  You probably do not want to #include this header directly.
- *
- *   Instead, it is preferred to #include "plmsys.h"
- */
-
 #include "plmsys_config.h"
+#include <string>
+
+class Dir_list_private;
 
 class PLMSYS_API Dir_list
 {
+public:
+    Dir_list_private *d_ptr;
 public:
     int num_entries;
     char** entries;
 public:
     Dir_list ();
     Dir_list (const char* dir);
+    Dir_list (const std::string& dir);
     ~Dir_list ();
 
     void init ();
     void load (const char* dir);
+    std::string entry (int idx);
 };
 
-/* logfile.cxx */
-PLMSYS_C_API void logfile_open (char* log_fn);
-PLMSYS_C_API void logfile_close (void);
-PLMSYS_C_API void logfile_printf (const char* fmt, ...);
-#define lprintf logfile_printf
-
-
 #endif
