@@ -95,6 +95,16 @@ static inline void vec3_add3 (double* v1, const double* v2, const double* v3) {
     v1[0] = v2[0] + v3[0]; v1[1] = v2[1] + v3[1]; v1[2] = v2[2] + v3[2];
 }
 
+template<class T> static inline void
+vec3_add (T* v1, const T* v2) {
+    v1[0] += v2[0]; v1[1] += v2[1]; v1[2] += v2[2];
+}
+
+template<class T> static inline void
+vec3_add (T* v1, const T* v2, const T* v3) {
+    v1[0] = v2[0] + v3[0]; v1[1] = v2[1] + v3[1]; v1[2] = v2[2] + v2[2];
+}
+
 template<class T> static inline void
 vec3_copy (T* v1, const T* v2) {
     v1[0] = v2[0]; v1[1] = v2[1]; v1[2] = v2[2];
@@ -274,6 +284,11 @@ plm_min (T v1, T v2) {
     return (v1 < v2) ? v1 : v2;
 }
 
+template<class T> T
+radians_from_degrees (T degrees) {
+    return degrees * M_PI / 180.;
+}
+
 // Cf. http://realtimecollisiondetection.net/blog/?p=89
 // https://randomascii.wordpress.com/2012/02/25/comparing-floating-point-numbers-2012-edition/
 static inline bool
