@@ -4,16 +4,9 @@
 #ifndef _print_and_exit_h_
 #define _print_and_exit_h_
 
-/**
-*  You probably do not want to #include this header directly.
- *
- *   Instead, it is preferred to #include "plmsys.h"
- */
-
 #include "plmsys_config.h"
 
-PLMSYS_C_API void print_and_wait (char* prompt_fmt, ...);
-PLMSYS_C_API void print_and_exit (char* prompt_fmt, ...);
+PLMSYS_API void print_and_exit (const char* prompt_fmt, ...);
 #define error_printf(fmt, ...) \
     fprintf (stderr, "\nplastimatch has encountered an issue.\n" \
              "file: %s (line:%i)\n" fmt, __FILE__, __LINE__,##__VA_ARGS__)
