@@ -7,11 +7,92 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include "itkImageRegionIteratorWithIndex.h"
+#include "vnl/vnl_random.h"
 
-#include "plmbase.h"
-#include "plmutil.h"
-
+#include "itk_directions.h"
+#include "itk_image_type.h"
+#include "itk_point.h"
+#include "logfile.h"
+#include "plm_image.h"
+#include "plm_image_header.h"
 #include "plm_math.h"
+#include "rt_study.h"
+#include "rtss.h"
+#include "segmentation.h"
+#include "synthetic_mha.h"
+
+class Synthetic_mha_parms_private {
+public:
+    vnl_random v;
+    float gabor_freq;
+    float gabor_proj[3];
+};
+
+
+Synthetic_mha_parms::Synthetic_mha_parms ()
+{
+    d_ptr = new Synthetic_mha_parms_private;
+
+    output_type = PLM_IMG_TYPE_ITK_FLOAT;
+    pattern = PATTERN_GAUSS;
+    input_fn = "";
+
+    for (int i = 0; i < 3; i++) {
+        spacing[i] = 5.0f;
+        dim[i] = 100;
+        origin[i] = 0.0f;
+        gauss_center[i] = 0.0f;
+        gauss_std[i] = 100.0f;
+        sphere_center[i] = 0.0f;
+        sphere_radius[i] = 50.0f;
+        donut_center[i] = 0.0f;
+        lung_tumor_pos[i] = 0.0f;
+        dose_center[i] = 0.0f;
+        cylinder_center[i] = 0.0f;
+        cylinder_radius[i] = 0.0f;
+    }
+    background = -1000.0f;
+    foreground = 0.0f;
+    background_alpha = 1.0f;
+    foreground_alpha = 1.0f;
+    m_want_ss_img = false;
+    m_want_dose_img = false;
+    image_normalization = NORMALIZATION_NONE;
+    rect_size[0] = -50.0f;
+    rect_size[1] = +50.0f;
+    rect_size[2] = -50.0f;
+    rect_size[3] = +50.0f;
+    rect_size[4] = -50.0f;
+    rect_size[5] = +50.0f;
+    donut_radius[0] = 50.0f;
+    donut_radius[1] = 50.0f;
+    donut_radius[2] = 20.0f;
+    donut_rings = 2;
+    grid_spacing[0] = 10;
+    grid_spacing[1] = 10;
+    grid_spacing[2] = 10;
+    penumbra = 5.0f;
+    dose_size[0] = -50.0f;
+    dose_size[1] = +50.0f;
+    dose_size[2] = -50.0f;
+    dose_size[3] = +50.0f;
+    dose_size[4] = -50.0f;
+    dose_size[5] = +50.0f;
+    num_multi_sphere = 33;
+    noise_mean = 0;
+    noise_std = 1.f;
+    gabor_use_k_fib = false;
+    gabor_k_fib[0] = 0;
+    gabor_k_fib[1] = 1;
+    gabor_k[0] = 1.;
+    gabor_k[1] = 0.;
+    gabor_k[2] = 0.;
+}
+
+Synthetic_mha_parms::~Synthetic_mha_parms ()
+{
+    delete d_ptr;
+}
 
 static void 
 synth_dose (
@@ -31,11 +112,11 @@ synth_dose (
 
     /* uniform central dose */
     if (p[0] >= parms->dose_size[0] + parms->penumbra
-     && p[0] <= parms->dose_size[1] - parms->penumbra
-     && p[1] >= parms->dose_size[2] + parms->penumbra
-     && p[1] <= parms->dose_size[3] - parms->penumbra
-     && p[2] >= parms->dose_size[4] 
-     && p[2] <= parms->dose_size[5])
+        && p[0] <= parms->dose_size[1] - parms->penumbra
+        && p[1] >= parms->dose_size[2] + parms->penumbra
+        && p[1] <= parms->dose_size[3] - parms->penumbra
+        && p[2] >= parms->dose_size[4] 
+        && p[2] <= parms->dose_size[5])
     {
         *intens = parms->foreground;
         *label = 1;
@@ -94,9 +175,9 @@ synth_dose (
         f1 = parms->foreground;
         if (x > x0 && x < x1 && y > y0 && y < y1) {
             *intens = ((f0)/((x1-x0)*(y1-y0)))*(x1-x)*(y1-y) +
-                      ((f0)/((x1-x0)*(y1-y0)))*(x-x0)*(y1-y) +
-                      ((f0)/((x1-x0)*(y1-y0)))*(x1-x)*(y-y0) +
-                      ((f1)/((x1-x0)*(y1-y0)))*(x-x0)*(y-y0);
+                ((f0)/((x1-x0)*(y1-y0)))*(x-x0)*(y1-y) +
+                ((f0)/((x1-x0)*(y1-y0)))*(x1-x)*(y-y0) +
+                ((f1)/((x1-x0)*(y1-y0)))*(x-x0)*(y-y0);
         }
         x = p[0];
         y = p[1];
@@ -108,9 +189,9 @@ synth_dose (
         f1 = parms->foreground;
         if (x > x0 && x < x1 && y > y0 && y < y1) {
             *intens = ((f0)/((x1-x0)*(y1-y0)))*(x1-x)*(y1-y) +
-                      ((f0)/((x1-x0)*(y1-y0)))*(x-x0)*(y1-y) +
-                      ((f1)/((x1-x0)*(y1-y0)))*(x1-x)*(y-y0) +
-                      ((f0)/((x1-x0)*(y1-y0)))*(x-x0)*(y-y0);
+                ((f0)/((x1-x0)*(y1-y0)))*(x-x0)*(y1-y) +
+                ((f1)/((x1-x0)*(y1-y0)))*(x1-x)*(y-y0) +
+                ((f0)/((x1-x0)*(y1-y0)))*(x-x0)*(y-y0);
         }
         x = p[0];
         y = p[1];
@@ -122,9 +203,9 @@ synth_dose (
         f1 = parms->foreground;
         if (x > x0 && x < x1 && y > y0 && y < y1) {
             *intens = ((f0)/((x1-x0)*(y1-y0)))*(x1-x)*(y1-y) +
-                      ((f1)/((x1-x0)*(y1-y0)))*(x-x0)*(y1-y) +
-                      ((f0)/((x1-x0)*(y1-y0)))*(x1-x)*(y-y0) +
-                      ((f0)/((x1-x0)*(y1-y0)))*(x-x0)*(y-y0);
+                ((f1)/((x1-x0)*(y1-y0)))*(x-x0)*(y1-y) +
+                ((f0)/((x1-x0)*(y1-y0)))*(x1-x)*(y-y0) +
+                ((f0)/((x1-x0)*(y1-y0)))*(x-x0)*(y-y0);
         }
         x = p[0];
         y = p[1];
@@ -136,9 +217,9 @@ synth_dose (
         f1 = parms->foreground;
         if (x > x0 && x < x1 && y > y0 && y < y1) {
             *intens = ((f1)/((x1-x0)*(y1-y0)))*(x1-x)*(y1-y) +
-                      ((f0)/((x1-x0)*(y1-y0)))*(x-x0)*(y1-y) +
-                      ((f0)/((x1-x0)*(y1-y0)))*(x1-x)*(y-y0) +
-                      ((f0)/((x1-x0)*(y1-y0)))*(x-x0)*(y-y0);
+                ((f0)/((x1-x0)*(y1-y0)))*(x-x0)*(y1-y) +
+                ((f0)/((x1-x0)*(y1-y0)))*(x1-x)*(y-y0) +
+                ((f0)/((x1-x0)*(y1-y0)))*(x-x0)*(y-y0);
         }
     }
 } /* z-direction */
@@ -151,10 +232,10 @@ synth_gauss (
     const Synthetic_mha_parms *parms
 )
 {
-    float f = 0;
+    double f = 0;
     for (int d = 0; d < 3; d++) {
-        float f1 = phys[d] - parms->gauss_center[d];
-        f1 = f1 / parms->gauss_std[d];
+        double f1 = phys[d] - (double) parms->gauss_center[d];
+        f1 = f1 / (double) parms->gauss_std[d];
         f += f1 * f1;
     }
     f = exp (-0.5 * f);            /* f \in (0,1] */
@@ -207,10 +288,14 @@ synth_rect (
         && phys[2] >= parms->rect_size[4] 
         && phys[2] <= parms->rect_size[5])
     {
-        *intens = parms->foreground;
+        *intens 
+            = (1 - parms->foreground_alpha) * (*intens) 
+            + parms->foreground_alpha * parms->foreground;
         *label = 1;
     } else {
-        *intens = parms->background;
+        *intens 
+            = (1 - parms->background_alpha) * (*intens) 
+            + parms->background_alpha * parms->background;
         *label = 0;
     }
 }
@@ -230,10 +315,14 @@ synth_sphere (
         f += f1 * f1;
     }
     if (f > 1.0) {
-        *intens = parms->background;
+        *intens 
+            = (1 - parms->background_alpha) * (*intens) 
+            + parms->background_alpha * parms->background;
         *label = 0;
     } else {
-        *intens = parms->foreground;
+        *intens 
+            = (1 - parms->foreground_alpha) * (*intens) 
+            + parms->foreground_alpha * parms->foreground;
         *label = 1;
     }
 }
@@ -261,203 +350,6 @@ synth_multi_sphere (
     }
 }
 
-static void 
-synth_enclosed_rect (
-    float *intens, 
-    unsigned char *label,
-    const FloatPoint3DType& phys, 
-    const Synthetic_mha_parms *parms
-)
-{
-    float f = 0.;
-    float p[56]={ 
-        5,5,5,    95,10,95, 1,
-        5,90,5,   95,95,95, 1,
-        5,5,5,    10,95,95, 1,
-        90,5,5,   95,95,95, 1,
-        5,5,5,    95,95,10, 1,
-        5,5,90,   95,95,95, 1, 
-        35,35,35, 70,70,70, parms->enclosed_intens_f1, 
-        20,20,20, 80,25,30, parms->enclosed_intens_f2
-    };
-
-    // center is at 0, size is ~200
-    // must specify dimension 200, origin -100, voxel spacing 1 in Slicer
-    for (int i=0;i<7*8;i++) {
-        if ((i%7)!=6) {
-            p[i]-=50;
-        }
-    }
-    
-    for (int i=0;i<8;i++)
-    {
-        
-        if (i==6) { 
-            p[7*i+0]+=parms->enclosed_xlat1[0]; 
-            p[7*i+1]+=parms->enclosed_xlat1[1];
-            p[7*i+2]+=parms->enclosed_xlat1[2];
-            p[7*i+3]+=parms->enclosed_xlat1[0]; 
-            p[7*i+4]+=parms->enclosed_xlat1[1];
-            p[7*i+5]+=parms->enclosed_xlat1[2];
-        }
-        if (i==7) { 
-            p[7*i+0]+=parms->enclosed_xlat2[0];
-            p[7*i+1]+=parms->enclosed_xlat2[1];
-            p[7*i+2]+=parms->enclosed_xlat2[2];
-            p[7*i+3]+=parms->enclosed_xlat2[0];
-            p[7*i+4]+=parms->enclosed_xlat2[1];
-            p[7*i+5]+=parms->enclosed_xlat2[2];
-        }
-        if (p[7*i+0]<phys[0] && phys[0]<p[7*i+3] &&
-            p[7*i+1]<phys[1] && phys[1]<p[7*i+4] &&
-            p[7*i+2]<phys[2] && phys[2]<p[7*i+5]) 
-        { f = p[6+7*i]; } 
-    }
-
-    *intens = (1 - f) * parms->background + f * parms->foreground;
-    *label = 0;
-}
-
-//box + ellipsoid inside
-static void
-synth_osd (
-    float *intens, 
-    unsigned char *label,
-    const FloatPoint3DType& phys, 
-    const Synthetic_mha_parms *parms
-)
-{
-    float f;
-    float p[36]={ 
-        5,5,5,  195,8,195, 
-        5,193,5, 195,195,195, 
-        5,5,5,  8,195,195, 
-        193,5,5, 195,195,195, 
-        5,5,5,  195,195,8, 
-        5,5,193, 195,195,195  
-    };
-
-    // must specify dimension 200, origin -100, voxel spacing 1 in Slicer
-    for (int i=0;i<36;i++) {
-        p[i]-=100;
-    } 
-                
-    f = 0.; *label=0;
-    for (int i=0;i<6;i++)
-    {
-
-        if (p[6*i+0]<phys[0] && phys[0]<p[6*i+3] &&
-            p[6*i+1]<phys[1] && phys[1]<p[6*i+4] &&
-            p[6*i+2]<phys[2] && phys[2]<p[6*i+5]) 
-        { f = 1.; *label=0; } 
-    }
-
-    // sphere
-    if (parms->pattern_ss == PATTERN_SS_ONE) {
-        float rs1= 25, rs2= 25, rs3=25;
-        float xs=101, ys=101, zs=101;
-
-        xs-=100; ys-=100; zs-=100; 
-
-        float rr = (phys[2]-zs)*(phys[2]-zs)/(rs1*rs1)
-            +(phys[1]-ys)*(phys[1]-ys)/(rs2*rs2)
-            +(phys[0]-xs)*(phys[0]-xs)/(rs3*rs3);
-
-        if ( rr < 2 ) { f = 1.; *label=1; }
-    }
-
-    // two spheres
-    if (parms->pattern_ss == PATTERN_SS_TWO_APART) {
-        float rs1= 25, rs2= 25, rs3=25;
-        float xs=-40, ys=1, zs=1;
-
-        float rr = (phys[2]-zs)*(phys[2]-zs)/(rs1*rs1)
-            +(phys[1]-ys)*(phys[1]-ys)/(rs2*rs2)
-            +(phys[0]-xs)*(phys[0]-xs)/(rs3*rs3);
-
-        if ( rr < 2 ) { f = 1.; *label=1;}
-
-        xs=40, ys=1, zs=1;
-
-        rr = (phys[2]-zs)*(phys[2]-zs)/(rs1*rs1)
-            +(phys[1]-ys)*(phys[1]-ys)/(rs2*rs2)
-            +(phys[0]-xs)*(phys[0]-xs)/(rs3*rs3);
-        
-        if ( rr < 2 ) { f = 1.; *label=2;}
-    }
-
-    // two spheres partially overlapping and one aside
-    if (parms->pattern_ss == PATTERN_SS_TWO_OVERLAP_PLUS_ONE) {
-        float rs1= 25, rs2= 25, rs3=25;
-        float xs=-20, ys=1, zs=1;
-
-        float rr = (phys[2]-zs)*(phys[2]-zs)/(rs1*rs1)
-            +(phys[1]-ys)*(phys[1]-ys)/(rs2*rs2)
-            +(phys[0]-xs)*(phys[0]-xs)/(rs3*rs3);
-
-        if ( rr < 2 ) { f = 1.; *label=1;}
-
-        xs=15, ys=25, zs=1;
-
-        rr = (phys[2]-zs)*(phys[2]-zs)/(rs1*rs1)
-            +(phys[1]-ys)*(phys[1]-ys)/(rs2*rs2)
-            +(phys[0]-xs)*(phys[0]-xs)/(rs3*rs3);
-        
-        if ( rr < 2 ) { f = 1.; *label=2;}
-
-        xs=30, ys=-55, zs=1;
-        rs1 = rs2 = rs3 = 10;
-
-        rr = (phys[2]-zs)*(phys[2]-zs)/(rs1*rs1)
-            +(phys[1]-ys)*(phys[1]-ys)/(rs2*rs2)
-            +(phys[0]-xs)*(phys[0]-xs)/(rs3*rs3);
-
-        if ( rr < 2 ) { f = 1.; *label=3;}
-
-    }
-
-    // two spheres partially overlapping, one within, and one aside
-    if (parms->pattern_ss == PATTERN_SS_TWO_OVERLAP_PLUS_ONE_PLUS_EMBED) {
-        float rs1= 25, rs2= 25, rs3=25;
-        float xs=-30, ys=1, zs=1;
-
-        float rr = (phys[2]-zs)*(phys[2]-zs)/(rs1*rs1)
-            +(phys[1]-ys)*(phys[1]-ys)/(rs2*rs2)
-            +(phys[0]-xs)*(phys[0]-xs)/(rs3*rs3);
-
-        if ( rr < 2 ) { f = 1.; *label=1;}
-
-        xs=5, ys=25, zs=1;
-
-        rr = (phys[2]-zs)*(phys[2]-zs)/(rs1*rs1)
-            +(phys[1]-ys)*(phys[1]-ys)/(rs2*rs2)
-            +(phys[0]-xs)*(phys[0]-xs)/(rs3*rs3);
-        
-        if ( rr < 2 ) { f = 1.; *label=2;}
-
-        xs=30, ys=-55, zs=1;
-        rs1 = rs2 = rs3 = 10;
-
-        rr = (phys[2]-zs)*(phys[2]-zs)/(rs1*rs1)
-            +(phys[1]-ys)*(phys[1]-ys)/(rs2*rs2)
-            +(phys[0]-xs)*(phys[0]-xs)/(rs3*rs3);
-
-        if ( rr < 2 ) { f = 1.; *label=3;}
-
-        xs=13, ys=33, zs=1;
-        rs1 = rs2 = rs3 = 8;
-
-        rr = (phys[2]-zs)*(phys[2]-zs)/(rs1*rs1)
-            +(phys[1]-ys)*(phys[1]-ys)/(rs2*rs2)
-            +(phys[0]-xs)*(phys[0]-xs)/(rs3*rs3);
-
-        if ( rr < 2 ) { f = 0.5; *label=4;}
-
-    }
-
-    *intens = (1 - f) * parms->background + f * parms->foreground;
-}
-
 static void 
 synth_donut (
     float *intens, 
@@ -530,6 +422,7 @@ synth_lung (
     /* If within chest wall, set chest wall intensity */
     if (d2 > 130) {
         *intens = parms->foreground;
+        *label = 1;
         return;
     }
 
@@ -543,90 +436,182 @@ synth_lung (
 
     /* If within tumor, set tumor density */
     if (d3 < 20) {
-        *label = 1;
+        *label = 3;
         *intens = parms->foreground;
         return;
     }
 
     /* Otherwise, must be lung */
     *intens = -700;
-    *label = 2;
+    *label = 5;
 }
 
-static float 
-shifttanh (float x)
-{
-    return 0.5* ((exp(x)-exp(-x))/(exp(x)+exp(-x)) + 1);
-}
-
-//synthetic dose distribution
-static float 
-intens_dosemha (
+static void 
+synth_ramp (
+    float *intens, 
+    unsigned char *label,
     const FloatPoint3DType& phys, 
-    float xlat1[3], 
-    float xlat2[3],
-    float f1, 
-    float f2, 
-    Pattern_structset_type pattern_ss
+    const Synthetic_mha_parms *parms
 )
 {
-    float f=0;
-    float x0=0, y0=0, z0=0;
-    float sigma=30;
 
-    if (pattern_ss == PATTERN_SS_ONE) { x0=1, y0=1, z0=1; }
-    if (pattern_ss == PATTERN_SS_TWO_APART) { x0=-40, y0=1, z0=1; }
-    if (pattern_ss == PATTERN_SS_TWO_OVERLAP_PLUS_ONE) { x0=-20, y0=1, z0=1; }
-    if (pattern_ss == PATTERN_SS_TWO_OVERLAP_PLUS_ONE_PLUS_EMBED) { x0=30, y0=-55, z0=1, sigma=12; }
+    /* Get distance from origin */
+    float d;
+    if (parms->pattern == PATTERN_XRAMP) {
+        d = phys[0] - parms->origin[0];
+    }
+    else if (parms->pattern == PATTERN_YRAMP) {
+        d = phys[1] - parms->origin[1];
+    }
+    else {
+        d = phys[2] - parms->origin[2];
+    }
 
-    float r = (phys[0]-x0)*(phys[0]-x0)+
-              (phys[1]-y0)*(phys[1]-y0)+
-              (phys[2]-z0)*(phys[2]-z0);
+    /* Set intensity */
+    *label = 0;
+    *intens = d;
+}
 
-    f  += exp(-r/(sigma*sigma));
+static void 
+synth_noise (
+    float *intens, 
+    unsigned char *label,
+    const FloatPoint3DType& phys, 
+    const Synthetic_mha_parms *parms
+)
+{
+    double r = parms->d_ptr->v.normal64();
+    
+    /* Set intensity */
+    *label = 0;
+    *intens = parms->noise_mean + (float) r * parms->noise_std;
+}
 
-    /*
-    r = (phys[0]-100.)*(phys[0]-100.)+
-        (phys[1]-100.)*(phys[1]-100.)+
-        (phys[2]-100.)*(phys[2]-100.)/(1.2*1.2);
 
-    f += exp(-r/(30*30));
-    */
-    f = 20 * shifttanh(2.5*f-0.3);
+static void 
+synth_cylinder (
+    float *intens, 
+    unsigned char *label,
+    const FloatPoint3DType& phys, 
+    const Synthetic_mha_parms *parms
+)
+{
+    float f = 0;
+    for (int d = 0; d < 2; d++) {
+        float f1 = phys[d] - parms->cylinder_center[d];
+        f1 = f1 / parms->cylinder_radius[d];
+        f += f1 * f1;
+    }
+    if (f > 1.0) {
+        *intens 
+            = (1 - parms->background_alpha) * (*intens) 
+            + parms->background_alpha * parms->background;
+        *label = 0;
+    } else {
+        *intens 
+            = (1 - parms->foreground_alpha) * (*intens) 
+            + parms->foreground_alpha * parms->foreground;
+        *label = 1;
+    }	
+}
 
-    return f;
+static void 
+synth_gabor (
+    float *intens, 
+    unsigned char *label,
+    const FloatPoint3DType& phys, 
+    const Synthetic_mha_parms *parms
+)
+{
+    /* Do gaussian first */
+    float f = 0;
+    float rel[3];
+    for (int d = 0; d < 3; d++) {
+        rel[d] = phys[d] - parms->gauss_center[d];
+        float f1 = rel[d] / parms->gauss_std[d];
+        f += f1 * f1;
+    }
+    f = exp (-0.5 * f);            /* f \in (0,1] */
+
+    *intens = (1 - f) * parms->background + f * parms->foreground;
+    *label = (f > 0.2) ? 1 : 0;
+
+    /* Get the distance from the central plane in direction of 
+       projection vector ("k" vector) */
+    float proj_dist = 0.f;
+    for (int d = 0; d < 3; d++) {
+        proj_dist += rel[d] * parms->d_ptr->gabor_proj[d];
+    }
+
+    /* Modulate by cos for real part, or sin for imaginary part */
+    *intens *= cos(parms->d_ptr->gabor_freq * proj_dist);
 }
 
 void
 synthetic_mha (
-    Rtds *rtds,
+    Rt_study *rt_study,
     Synthetic_mha_parms *parms
 )
 {
-    /* Create ITK images for intensity, ss, and dose */
     FloatImageType::SizeType sz;
     FloatImageType::IndexType st;
     FloatImageType::RegionType rg;
     FloatImageType::PointType og;
     FloatImageType::SpacingType sp;
     FloatImageType::DirectionType itk_dc;
-    for (int d1 = 0; d1 < 3; d1++) {
-        st[d1] = 0;
-        sz[d1] = parms->dim[d1];
-        sp[d1] = parms->spacing[d1];
-        og[d1] = parms->origin[d1];
+    FloatImageType::Pointer im_out;
+
+    if (parms->input_fn != "") {
+        /* Input image was specified */
+        Plm_image pi (parms->input_fn);
+        im_out = pi.itk_float ();
+
+        /* GCS FIX: Ideally, the calling code will set the alpha values 
+           properly.  Instead, here we set the background alpha to 0, 
+           with the understanding that the caller probably wants to 
+           paste a sphere or rectangle onto the existing image */
+        parms->background_alpha = 0.0f;
+
+    } else {
+        /* No input image specified */
+        if (parms->fixed_fn != "") {
+            /* Get geometry from fixed image */
+            Plm_image pi (parms->fixed_fn);
+            Plm_image_header pih;
+            pih.set_from_plm_image (&pi);
+            og = pih.GetOrigin();
+            sp = pih.GetSpacing();
+            rg = pih.GetRegion();
+            itk_dc = pih.GetDirection();
+            for (int d1 = 0; d1 < 3; d1++) {
+                parms->origin[d1] = og[d1];
+            }
+        } else {
+            /* Get geometry from command line parms */
+            for (int d1 = 0; d1 < 3; d1++) {
+                st[d1] = 0;
+                sz[d1] = parms->dim[d1];
+                sp[d1] = parms->spacing[d1];
+                og[d1] = parms->origin[d1];
+            }
+            rg.SetSize (sz);
+            rg.SetIndex (st);
+            itk_direction_from_dc (&itk_dc, parms->dc);
+        }
+
+        /* Create new ITK image for intensity */
+        im_out = FloatImageType::New();
+        im_out->SetRegions (rg);
+        im_out->SetOrigin (og);
+        im_out->SetSpacing (sp);
+        im_out->SetDirection (itk_dc);
+        im_out->Allocate ();
+
+        /* Initialize to background */
+        im_out->FillBuffer (parms->background);
     }
-    rg.SetSize (sz);
-    rg.SetIndex (st);
-    itk_direction_from_dc (&itk_dc, parms->dc);
-
-    FloatImageType::Pointer im_out = FloatImageType::New();
-    im_out->SetRegions (rg);
-    im_out->SetOrigin (og);
-    im_out->SetSpacing (sp);
-    im_out->SetDirection (itk_dc);
-    im_out->Allocate();
 
+    /* Create new ITK images for ss and dose */
     UCharImageType::Pointer ss_img = UCharImageType::New();
     typedef itk::ImageRegionIteratorWithIndex< UCharImageType > 
         UCharIteratorType;
@@ -656,12 +641,61 @@ synthetic_mha (
         dose_img_it.GoToBegin();
     }
 
+    /* Figure out Gabor settings */
+    if (parms->pattern == PATTERN_GABOR) {
+        /* Figure out projection direction */
+        if (parms->gabor_use_k_fib) {
+            /* Fibonacci method selected. */
+            /* N.b. I'm not sure which is the better value for phi.
+               So let's use the golden section value. */
+            //float phi = (3. - sqrt(5.)) * M_PI;
+            float phi = (((sqrt(5.) + 1.) / 2.) - 1.) * M_PI;
+            float longitude = fmodf (phi * parms->gabor_k_fib[0], 2 * M_PI);
+            float latitude_sin = -1. + 2. * ((float) parms->gabor_k_fib[0]) / 
+                parms->gabor_k_fib[1];
+            float latitude = asinf (latitude_sin);
+            float latitude_cos = cos (latitude);
+            parms->d_ptr->gabor_proj[0] = latitude_cos * cos (longitude);
+            parms->d_ptr->gabor_proj[1] = latitude_cos * sin (longitude);
+            parms->d_ptr->gabor_proj[2] = latitude_sin;
+            
+            lprintf ("Gabor: %d %d -> (%f %f) -> (%f %f %f)\n", 
+                parms->gabor_k_fib[0],
+                parms->gabor_k_fib[1],
+                longitude, latitude, 
+                parms->d_ptr->gabor_proj[0],
+                parms->d_ptr->gabor_proj[1],
+                parms->d_ptr->gabor_proj[2]);
+        }
+        else {
+            /* Standard "k" vector supplied.  Need to normalize it. */
+            float len = vec3_len (parms->gabor_k);
+            if (len < FLOAT_SMALL_VECTOR_LENGTH) {
+                parms->gabor_k[0] = 1.;
+                parms->gabor_k[1] = 0.;
+                parms->gabor_k[2] = 0.;
+                len = 1.;
+            }
+            vec3_scale3 (parms->d_ptr->gabor_proj, parms->gabor_k, len);
+        }
+
+        /* Automatically set frequency based on width.  
+           The value 1.5 is a heuristic. */
+        float sum = 0.f;
+        for (int d = 0; d < 3; d++) {
+            sum += parms->gauss_std[d] * parms->gauss_std[d];
+        }
+        parms->d_ptr->gabor_freq = M_PI * 1.5 / sqrt(sum);
+        parms->image_normalization = Synthetic_mha_parms::NORMALIZATION_GABOR;
+    }
+
     /* Iterate through image, setting values */
     typedef itk::ImageRegionIteratorWithIndex< FloatImageType > IteratorType;
     IteratorType it_out (im_out, im_out->GetLargestPossibleRegion());
     for (it_out.GoToBegin(); !it_out.IsAtEnd(); ++it_out) {
         FloatPoint3DType phys;
-        float intens = 0.0f;
+        //float intens = 0.0f;
+        float intens = it_out.Get();
         unsigned char label_uchar = 0;
 
         /* Get 3D coordinates of voxel */
@@ -682,12 +716,6 @@ synthetic_mha (
         case PATTERN_MULTI_SPHERE:
             synth_multi_sphere (&intens, &label_uchar, phys, parms);
             break;
-        case PATTERN_ENCLOSED_RECT:
-            synth_enclosed_rect (&intens, &label_uchar, phys, parms);
-            break;
-        case PATTERN_OBJSTRUCTDOSE:
-            synth_osd (&intens, &label_uchar, phys, parms);
-            break;
         case PATTERN_DONUT:
             synth_donut (&intens, &label_uchar, phys, parms);
             break;
@@ -700,6 +728,20 @@ synthetic_mha (
         case PATTERN_LUNG:
             synth_lung (&intens, &label_uchar, phys, parms);
             break;
+        case PATTERN_XRAMP:
+        case PATTERN_YRAMP:
+        case PATTERN_ZRAMP:
+            synth_ramp (&intens, &label_uchar, phys, parms);
+            break;
+        case PATTERN_NOISE:
+            synth_noise (&intens, &label_uchar, phys, parms);
+            break;
+        case PATTERN_CYLINDER:
+            synth_cylinder (&intens, &label_uchar, phys, parms);
+            break;
+        case PATTERN_GABOR:
+            synth_gabor (&intens, &label_uchar, phys, parms);
+            break;
         default:
             intens = 0.0f;
             label_uchar = 0;
@@ -718,41 +760,139 @@ synthetic_mha (
         /* Set dose */
         if (parms->m_want_dose_img) {
             float dose = 0.;
-            if (parms->pattern == PATTERN_OBJSTRUCTDOSE) {
-                dose = intens_dosemha (phys,
-                    parms->enclosed_xlat1, parms->enclosed_xlat2,
-                    parms->enclosed_intens_f1, parms->enclosed_intens_f2, 
-                    parms->pattern_ss);
+            const float thresh = parms->background + 
+                0.5 * (parms->foreground - parms->background);
+            if (parms->foreground > parms->background 
+                && intens > thresh)
+            {
+                dose = 15;
+            } else if (parms->foreground < parms->background 
+                && intens < thresh)
+            {
+                dose = 15;
             } else {
-                const float thresh = parms->background + 
-                    0.5 * (parms->foreground - parms->background);
-                if (parms->foreground > parms->background 
-                    && intens > thresh)
-                {
-                    dose = 15;
-                } else if (parms->foreground < parms->background 
-                    && intens < thresh)
-                {
-                    dose = 15;
-                } else {
-                    dose = 0;
-                }
+                dose = 0;
             }
             dose_img_it.Set (dose);
             ++dose_img_it;
         }
     }
 
-    /* Insert images into rtds */
-    rtds->m_img = new Plm_image;
-    rtds->m_img->set_itk (im_out);
+    /* Normalize if requested */
+    if (parms->image_normalization == Synthetic_mha_parms::NORMALIZATION_SUM_ONE) {
+        float sum = 0.f;
+        for (it_out.GoToBegin(); !it_out.IsAtEnd(); ++it_out) {
+            sum += it_out.Get();
+        }
+        if (sum > 1e-10) {
+            for (it_out.GoToBegin(); !it_out.IsAtEnd(); ++it_out) {
+                it_out.Set(it_out.Get() / sum);
+            }
+        }
+    }
+    else if (parms->image_normalization == Synthetic_mha_parms::NORMALIZATION_SUM_SQR_ONE) {
+        float sum = 0.f;
+        for (it_out.GoToBegin(); !it_out.IsAtEnd(); ++it_out) {
+            float v = it_out.Get();
+            sum += v * v;
+        }
+        sum = sqrt(sum);
+        if (sum > 1e-10) {
+            for (it_out.GoToBegin(); !it_out.IsAtEnd(); ++it_out) {
+                it_out.Set(it_out.Get() / sum);
+            }
+        }
+    }
+    else if (parms->image_normalization == Synthetic_mha_parms::NORMALIZATION_ZERO_MEAN_STD_ONE) {
+        /* GCS FIX: I should really compute in a single pass.
+           Oh God almighty, when did I get so lazy? */
+        int num_vox = 0;
+        float sum_1 = 0.f, sum_2 = 0.f;
+        for (it_out.GoToBegin(); !it_out.IsAtEnd(); ++it_out) {
+            float v = it_out.Get();
+            sum_1 += v;
+            num_vox ++;
+        }
+        sum_1 = sum_1 / num_vox;
+        for (it_out.GoToBegin(); !it_out.IsAtEnd(); ++it_out) {
+            float v = it_out.Get() - sum_1;
+            it_out.Set(v);
+            sum_2 += v * v;
+        }
+        sum_2 = sqrt(sum_2);
+        if (sum_2 > 1e-10) {
+            for (it_out.GoToBegin(); !it_out.IsAtEnd(); ++it_out) {
+                it_out.Set(it_out.Get() / sum_2);
+            }
+        }
+    }
+    else if (parms->image_normalization == Synthetic_mha_parms::NORMALIZATION_GABOR) {
+        float sum_lo_1 = 0.f, sum_hi_1 = 0.f;
+        float sum_lo_2 = 0.f, sum_hi_2 = 0.f;
+        for (it_out.GoToBegin(); !it_out.IsAtEnd(); ++it_out) {
+            float v = it_out.Get();
+            if (v < 0) {
+                sum_lo_1 += v;
+                sum_lo_2 += v * v;
+            } else if (v > 0) {
+                sum_hi_1 += v;
+                sum_hi_2 += v * v;
+            }
+        }
+        sum_lo_2 = M_SQRT1_2 * sqrt(sum_lo_2);
+        sum_hi_2 = M_SQRT1_2 * sqrt(sum_hi_2);
+        float sum_lo_3 = 0.f, sum_hi_3 = 0.f;
+        if (sum_lo_2 > 1e-10 && sum_hi_2 > 1e-10) {
+            if (sum_lo_2 < sum_hi_2) {
+                sum_lo_2 = sum_hi_2 * -sum_lo_1 / sum_hi_1;
+            } else {
+                sum_hi_2 = sum_lo_2 * -sum_hi_1 / sum_lo_1;
+            }
+            for (it_out.GoToBegin(); !it_out.IsAtEnd(); ++it_out) {
+                float v = it_out.Get();
+                if (v < 0) {
+                    it_out.Set(v / sum_lo_2);
+                    sum_lo_3 += v / sum_lo_2;
+                } else if (v > 0) {
+                    it_out.Set(v / sum_hi_2);
+                    sum_hi_3 += v / sum_hi_2;
+                }
+            }
+        }
+    }
+
+    /* Insert images into rt_study */
+    Plm_image::Pointer pli = Plm_image::New();
+    pli->set_itk (im_out);
+    rt_study->set_image (pli);
     if (parms->m_want_ss_img) {
-        rtds->m_rtss = new Rtss (rtds);
-        rtds->m_rtss->m_ss_img = new Plm_image;
-        rtds->m_rtss->m_ss_img->set_itk (ss_img);
+        /* Create rtss & set into rt_study */
+        Segmentation::Pointer seg = Segmentation::New ();
+        rt_study->set_segmentation (seg);
+
+        /* Insert ss_image into rtss */
+        seg->set_ss_img (ss_img);
+
+        /* Insert structure set into rtss */
+        Rtss *rtss_ss = new Rtss;
+        seg->set_structure_set (rtss_ss);
+
+        /* Add structure names */
+        switch (parms->pattern) {
+        case PATTERN_LUNG:
+            rtss_ss->add_structure (
+                std::string ("Body"), std::string(), 1, 0);
+            rtss_ss->add_structure (
+                std::string ("Tumor"), std::string(), 2, 1);
+            rtss_ss->add_structure (
+                std::string ("Lung"), std::string(), 3, 2);
+            break;
+        default:
+            rtss_ss->add_structure (
+                std::string ("Foreground"), std::string(), 1, 0);
+        }
     }
     if (parms->m_want_dose_img) {
-        rtds->m_dose = new Plm_image;
-        rtds->m_dose->set_itk (dose_img);
+        rt_study->set_dose (dose_img);
     }
 }
